@@ -1,9 +1,9 @@
 # pylint: skip-file
 from __future__ import annotations
 
+import json
 from dataclasses import dataclass
 from typing import Dict, List
-
 from .model import StrEnum
 
 
@@ -115,6 +115,7 @@ class Configuration:
     loggingConfiguration: LoggingConfig
     logCompoundInteractions: bool
     testCaseSplitPathRegEx: str
+    phasePattern: str
 
     @classmethod
     def from_dict(cls, dictionary) -> Configuration:
@@ -135,5 +136,18 @@ class Configuration:
             ),
             logCompoundInteractions=dictionary.get("logCompoundInteractions", True),
             resourceDirectory=dictionary.get("resourceDirectory", "{root}/Resources"),
-            testCaseSplitPathRegEx=dictionary.get("testCaseSplitPathRegEx", "^<regex>$"),
+            testCaseSplitPathRegEx=dictionary.get(
+                "testCaseSplitPathRegEx", ".*StopWithRestart.*"
+            ),
+            phasePattern=dictionary.get("phasePattern", "{testcase} : Phase {index}/{length}"),
+        )
+
+
+def write_default_config(config_file):
+    with open(config_file, 'w', encoding='utf-8') as file:
+        json.dump(
+            Configuration.from_dict({}).__dict__,
+            file,
+            default=lambda o: o.__dict__,
+            indent=2,
         )
