@@ -1,25 +1,26 @@
 from __future__ import annotations
-from dataclasses import dataclass
-from pathlib import PurePath, Path
-from typing import Dict, List, Set, Optional
+
 import os
 import re
+from dataclasses import dataclass
+from pathlib import Path, PurePath
+from typing import Dict, List, Optional, Set
 
 from robot.parsing.lexer.tokens import Token
-from robot.parsing.model.blocks import SettingSection, TestCase, TestCaseSection, File
+from robot.parsing.model.blocks import File, SettingSection, TestCase, TestCaseSection
 from robot.parsing.model.statements import (
+    Comment,
     EmptyLine,
+    ForceTags,
     KeywordCall,
     LibraryImport,
-    ResourceImport,
-    VariablesImport,
     Metadata,
+    ResourceImport,
     SectionHeader,
-    TestCaseName,
-    Comment,
     Statement,
-    ForceTags,
     Tags,
+    TestCaseName,
+    VariablesImport,
 )
 
 from .config import Configuration
@@ -27,18 +28,19 @@ from .json_reader import TestCaseSet
 from .log import logger
 from .model import (
     InteractionDetails,
-    TestCaseDetails,
-    TestStructureTree,
     InteractionType,
     ParameterUseType,
+    TestCaseDetails,
     TestStructureTreeNode,
-    UserDefinedField,
     UdfType,
+    UserDefinedField,
 )
 from .utils import PathResolver
 
-
-SEPERATOR = "    "
+SEPARATOR = "    "
+ROBOT_PATH_SEPARATOR = "/"
+SECTION_SEPARATOR = [EmptyLine.from_params()] * 2
+TESTCASE_SEPARATOR = [EmptyLine.from_params()]
 
 
 @dataclass
@@ -69,12 +71,15 @@ class RfTestCase:
         self.config = config
         for interaction in test_case_details.interactions:
             self._get_interaction_calls(interaction)
-        self.rf_tags = [
-            keyword.name for keyword in test_case_details.spec.keywords
-        ] + self._get_udf_tags(test_case_details.spec.udfs)
-
+        self.rf_tags = self._get_tags(test_case_details)
         # TODO description
 
+    @staticmethod
+    def _get_tags(test_case_details: TestCaseDetails) -> List[str]:
+        tags = [keyword.name for keyword in test_case_details.spec.keywords]
+        tags.extend([udf.robot_tag for udf in test_case_details.spec.udfs if udf.robot_tag])
+        return tags
+
     @staticmethod
     def _get_udf_tags(user_defined_fields: List[UserDefinedField]) -> List[str]:
         udfs = []
@@ -97,11 +102,13 @@ class RfTestCase:
                 ParameterUseType.CallByReferenceMandatory,
             )
             if interaction.interactionType == InteractionType.Compound:
-                self._append_compound_ia_and_analyze_childs(
+                self._append_compound_ia_and_analyze_children(
                     cbr_params, cbv_params, indent, interaction
                 )
             elif interaction.interactionType == InteractionType.Atomic:
-                self._append_atomic_ia(cbr_params, cbv_params, indent, interaction)
+                self._append_atomic_ia(
+                    cbr_params, cbv_params, indent, interaction
+                )  # TODO: Else für textuelle Interaktionen
 
     def _append_atomic_ia(
         self,
@@ -112,14 +119,14 @@ class RfTestCase:
     ):
         ia_path_parts = interaction.path.split(".")
         if len(ia_path_parts) == 1:
-            root_subdivion = "UNKNOWN"
+            root_subdivision = "UNKNOWN"
             import_prefix = ia_path_parts[0]
         else:
-            root_subdivion, import_prefix = ia_path_parts[:2]
-        if root_subdivion not in self.used_imports:
-            self.used_imports[root_subdivion] = {import_prefix}
+            root_subdivision, import_prefix = ia_path_parts[:2]
+        if root_subdivision not in self.used_imports:
+            self.used_imports[root_subdivision] = {import_prefix}
         else:
-            self.used_imports[root_subdivion].add(import_prefix)
+            self.used_imports[root_subdivision].add(import_prefix)
         self.interaction_calls.append(
             AtomicInteractionCall(
                 name=interaction.name,
@@ -130,7 +137,7 @@ class RfTestCase:
             )
         )
 
-    def _append_compound_ia_and_analyze_childs(
+    def _append_compound_ia_and_analyze_children(
         self,
         cbr_params: Dict[str, str],
         cbv_params: Dict[str, str],
@@ -175,21 +182,34 @@ class RfTestCase:
             and keyword_lists[tc_index]
         )
 
-    def to_robot_ast_test_cases(self) -> List[TestCase]:
+    def to_robot_ast_test_cases(
+        self,
+    ) -> List[
+        TestCase
+    ]:  # TODO: Separate testcase splitting from this method --> new method: to_robot_ast_test_case
         rf_keyword_lists = self._create_rf_keywords(self.interaction_calls)
         rf_test_cases: List[TestCase] = []
         multiple_tests = len(rf_keyword_lists) > 1
         for index, rf_keywords in enumerate(rf_keyword_lists):
-            suffix = f' : Phase {index + 1}/{len(rf_keyword_lists)}' if multiple_tests else ''
-            tc_name = f"{self.uid}{suffix}"  # TODO later UID or Comments
+            phase_pattern = self.config.phasePattern
+            tc_name = (
+                phase_pattern.format(
+                    testcase=self.uid, index=index + 1, length=len(rf_keyword_lists)
+                )
+                if multiple_tests
+                else self.uid
+            )
+            # suffix = f' : Phase {index + 1}/{len(rf_keyword_lists)}' if multiple_tests else ''
+            # tc_name = f"{self.uid}{suffix}"  # TODO later UID or Comments
             rf_test_case = TestCase(header=TestCaseName.from_params(tc_name))
             rf_test_case.body.append(Tags.from_params(self.rf_tags))
             rf_test_case.body.extend(rf_keywords)
-            rf_test_case.body.extend([EmptyLine.from_params()])
+            rf_test_case.body.extend(TESTCASE_SEPARATOR)
             rf_test_cases.append(rf_test_case)
         return rf_test_cases
 
-    def _create_cbv_parameters(self, interaction: AtomicInteractionCall) -> List[str]:
+    @staticmethod
+    def _create_cbv_parameters(interaction: AtomicInteractionCall) -> List[str]:
         parameters = []
         for name, value in interaction.cbv_parameters.items():
             if name.endswith('='):
@@ -201,7 +221,7 @@ class RfTestCase:
     def _create_rf_keyword(self, interaction: AtomicInteractionCall) -> KeywordCall:
         import_prefix = self.config.fullyQualified * f"{interaction.import_prefix}."
         interaction_indent = (
-            SEPERATOR * interaction.indent if self.config.logCompoundInteractions else SEPERATOR
+            SEPARATOR * interaction.indent if self.config.logCompoundInteractions else SEPARATOR
         )
         cbv_parameters = self._create_cbv_parameters(interaction)
         return KeywordCall.from_params(
@@ -220,21 +240,21 @@ class RfTestCase:
 
     @staticmethod
     def _generate_compound_interaction_comment(interaction: CompoundInteractionCall) -> str:
-        cbr_params = SEPERATOR.join(
+        cbr_params = SEPARATOR.join(
             [
                 f"{param_name}={param_value}"
                 for param_name, param_value in interaction.cbr_parameters.items()
             ]
         )
-        cbv_params = SEPERATOR.join(
+        cbv_params = SEPARATOR.join(
             [
                 f"{param_name}={param_value}"
                 for param_name, param_value in interaction.cbv_parameters.items()
             ]
         )
         if cbr_params:
-            return f"#{SEPERATOR.join([cbr_params, interaction.name, cbv_params])}"
-        return f"#{SEPERATOR.join([interaction.name, cbv_params])}"
+            return f"#{SEPARATOR.join([cbr_params, interaction.name, cbv_params])}"
+        return f"#{SEPARATOR.join([interaction.name, cbv_params])}"
 
     @staticmethod
     def _get_params_by_use_type(
@@ -249,12 +269,9 @@ class RfTestCase:
 
 def create_test_suites(
     test_case_set_catalog: Dict[str, TestCaseSet],
-    test_theme_tree: TestStructureTree,
+    path_resolver: PathResolver,
     config: Configuration,
 ) -> Dict[str, File]:
-    path_resolver = PathResolver(
-        test_theme_tree, tuple(test_case_set_catalog.keys()), config.logSuiteNumbering
-    )
     tcs_paths = path_resolver.tcs_paths
     test_suites = {}
     for uid, test_case_set in test_case_set_catalog.items():
@@ -282,11 +299,11 @@ class RobotInitFileBuilder:
         return File(sections, source=os.path.join(str(self.tt_path), "__init__"))
 
     def _create_setting_section(self) -> SettingSection:
-        setting_section_meta_data = self._get_setting_section_metadata()
         setting_section = SettingSection(header=SectionHeader.from_params(Token.SETTING_HEADER))
+        setting_section_meta_data = self._get_setting_section_metadata()
         setting_section.body.extend(
             [
-                Metadata.from_params(metadata_name, metadata_value)
+                create_meta_data(metadata_name, metadata_value)
                 for metadata_name, metadata_value in setting_section_meta_data.items()
             ]
         )
@@ -294,14 +311,26 @@ class RobotInitFileBuilder:
 
     def _get_setting_section_metadata(self) -> Dict[str, str]:
         meta_data = {
-            "uniqueID": self.test_theme.baseInformation.uniqueID,
-            "numbering": self.test_theme.baseInformation.numbering,
+            "UniqueID": self.test_theme.baseInformation.uniqueID,
+            "Numbering": self.test_theme.baseInformation.numbering,
         }
         if self.test_theme.specification:
-            meta_data["specification status"] = self.test_theme.specification.status
+            meta_data["Specification Status"] = self.test_theme.specification.status
         return meta_data
 
 
+def create_meta_data(name, value):
+    tokens = [
+        Token(Metadata, 'Metadata', 1),
+        Token(SEPARATOR, '    ', 2),
+        Token('NAME', name, 3),
+        Token(SEPARATOR, '    ', 4),
+        Token('ARGUMENT', value, 5),
+        Token('EOL', '\n', 6),
+    ]
+    return Metadata(tokens)
+
+
 class RobotSuiteFileBuilder:
     def __init__(
         self, test_case_set: TestCaseSet, tcs_path: PurePath, config: Configuration
@@ -324,17 +353,17 @@ class RobotSuiteFileBuilder:
         for test_case in self._rf_test_cases:
             robot_ast_test_cases.extend(test_case.to_robot_ast_test_cases())
         test_case_section.body.extend(robot_ast_test_cases)
-        test_case_section.body.extend([EmptyLine.from_params(), EmptyLine.from_params()])
+        test_case_section.body.extend(SECTION_SEPARATOR)
         return test_case_section
 
     def _get_used_subdivisions(self) -> Dict[str, Set[str]]:
         import_dict: Dict[str, Set[str]] = {}
         for test_case in self._rf_test_cases:
-            for root, import_name in test_case.used_imports.items():
+            for root, import_names in test_case.used_imports.items():
                 if root not in import_dict:
-                    import_dict[root] = {*import_name}
+                    import_dict[root] = import_names
                 else:
-                    import_dict[root].update(import_name)
+                    import_dict[root].update(import_names)
         return import_dict
 
     def _create_rf_variable_imports(self) -> List[VariablesImport]:
@@ -344,24 +373,29 @@ class RobotSuiteFileBuilder:
         ]
 
     def _create_rf_resource_imports(self, import_dict: Dict[str, Set[str]]) -> List[ResourceImport]:
-        resource_imports: Set[str] = set()
-        for resource_root in self.config.rfResourceRoots:
-            resource_imports.update(import_dict.pop(resource_root, []))
-        resource_imports.update(self.config.forcedImport.resources)
-        resource_files = {
-            self._create_resource_import(resource_import)
-            for resource_import in sorted(resource_imports)
-        }  # TODO Fix Paths to correct models
-        return [ResourceImport.from_params(res) for res in sorted(resource_files)]
+        resources = {
+            str(resource)
+            for resource_root, resources in import_dict.items() for resource in resources
+            if self._is_resource(resource_root)
+        }
 
-    def _create_resource_import(self, resource_import: str) -> str:
+        resources.update(self.config.forcedImport.resources)
+        resource_paths = {
+            self._create_resource_path(resource) for resource in sorted(resources)
+        }  # TODO Fix Paths to correct models
+        return [ResourceImport.from_params(res) for res in sorted(resource_paths)]
+
+    def _create_resource_path(self, resource: str) -> str:
         if not self.config.resourceDirectory:
-            return f"{resource_import}.resource"
+            return f"{resource}.resource"
         resource_dir = self._get_resource_directory()
-        resource_mapping = self.config.subdivisionsMapping.resources.get(resource_import)
-        if resource_mapping:
-            return f"{'../' * (len(self.tcs_path.parts))}{resource_dir}/{resource_mapping}"
-        return f"{'../' * (len(self.tcs_path.parts))}{resource_dir}/{resource_import}.resource"
+        resource_path = self.config.subdivisionsMapping.resources.get(resource)
+        resource_base_path = (
+            f"{('..' + ROBOT_PATH_SEPARATOR) * len(self.tcs_path.parts)}{resource_dir}"
+        )
+        if resource_path:
+            return f"{resource_base_path}{ROBOT_PATH_SEPARATOR}{resource_path}"
+        return f"{resource_base_path}{ROBOT_PATH_SEPARATOR}{resource}.resource"
 
     def _get_resource_directory(self) -> str:
         root_path = Path(os.curdir).absolute()
@@ -373,38 +407,45 @@ class RobotSuiteFileBuilder:
                 flags=re.IGNORECASE,
             ),
             root_path,
-        ).replace('\\', '/')
+        ).replace('\\', ROBOT_PATH_SEPARATOR)
+
+    def _is_library(self, root_subdivision: str) -> bool:
+        if root_subdivision in self.config.rfLibraryRoots:
+            return True
+        return False
+
+    def _is_resource(self, root_subdivision: str) -> bool:
+        if root_subdivision in self.config.rfResourceRoots:
+            return True
+        return False
 
     def _create_rf_library_imports(self, import_dict: Dict[str, Set[str]]) -> List[LibraryImport]:
-        lib_imports = set()
-        for lib_root in self.config.rfLibraryRoots:
-            lib_imports.update(import_dict.pop(lib_root, set()))
-        lib_imports.update(self.config.forcedImport.libraries)
         libraries = {
-            self.config.subdivisionsMapping.libraries.get(lib_import, lib_import)
-            for lib_import in lib_imports
+            next(iter(library))
+            for library_root, library in import_dict.items()
+            if self._is_library(library_root)
         }
-        return [LibraryImport.from_params(lib) for lib in sorted(libraries)]
+        libraries.update(self.config.forcedImport.libraries)
+        lib_imports = {
+            self.config.subdivisionsMapping.libraries.get(library, library) for library in libraries
+        }
+        return [LibraryImport.from_params(lib) for lib in sorted(lib_imports)]
 
     def _create_rf_force_tags(self) -> Optional[ForceTags]:
         tb_keyword_names = [keyword.name for keyword in self.test_case_set.details.spec.keywords]
-        udfs = []
-        for udf in self.test_case_set.details.spec.udfs:
-            if udf.valueType == UdfType.Enumeration:
-                udfs.append(f"{udf.name}:{udf.value}")
-            elif udf.valueType == UdfType.String and udf.value:
-                udfs.append(f"{udf.name}:{udf.value}")
-            elif udf.valueType == UdfType.Boolean and udf.value == "true":
-                udfs.append(udf.name)
+        udfs = [udf.robot_tag for udf in self.test_case_set.details.spec.udfs if udf.robot_tag]
         force_tags = tb_keyword_names + udfs
         if force_tags:
             return ForceTags.from_params(force_tags)
         return None
 
     def _create_rf_unknown_imports(self, import_dict: Dict[str, Set[str]]) -> List[Comment]:
-        unknown_imports = set()
+        unknown_imports = {
+            root_subdivision: next(iter(unknown_import))
+            for root_subdivision, unknown_import in import_dict.items()
+            if not self._is_library(root_subdivision) and not self._is_resource(root_subdivision)
+        }
         for root, subdivision_names in import_dict.items():
-            unknown_imports.update(subdivision_names)
             logger.debug(
                 f"{self.test_case_set.details.uniqueID} has imports {list(subdivision_names)} "
                 f"from unknown root subdivision '{root}'!"
@@ -412,6 +453,8 @@ class RobotSuiteFileBuilder:
         if unknown_imports:
             logger.warning(
                 f"{self.test_case_set.details.uniqueID} has unknown imports. "
+                f"TestBench Subdivisions which correspond to Libraries or Resources "
+                f"need to be declared in the configuration file. "
                 f"See Log for more details."
             )
         return [
@@ -422,25 +465,18 @@ class RobotSuiteFileBuilder:
     def _create_setting_section(self) -> SettingSection:
         subdivisions = self._get_used_subdivisions()
 
-        setting_section_meta_data = self._get_setting_section_metadata()
         setting_section = SettingSection(header=SectionHeader.from_params(Token.SETTING_HEADER))
         setting_section.body.extend(self._create_rf_variable_imports())
         setting_section.body.extend(self._create_rf_library_imports(subdivisions))
         setting_section.body.extend(self._create_rf_resource_imports(subdivisions))
         setting_section.body.extend(self._create_rf_unknown_imports(subdivisions))
+        setting_section_meta_data = self.test_case_set.metadata
         setting_section.body.extend(
             [
-                Metadata.from_params(metadata_name, metadata_value)
+                create_meta_data(metadata_name, metadata_value)
                 for metadata_name, metadata_value in setting_section_meta_data.items()
             ]
         )
         setting_section.body.append(self._create_rf_force_tags())
-        setting_section.body.extend([EmptyLine.from_params(), EmptyLine.from_params()])
+        setting_section.body.extend(SECTION_SEPARATOR)
         return setting_section
-
-    def _get_setting_section_metadata(self) -> Dict[str, str]:
-        return {
-            "uniqueID": self.test_case_set.details.uniqueID,
-            "name": self.test_case_set.details.name,
-            "numbering": self.test_case_set.details.numbering,
-        }
